;POOL classes for elastically scattering turtles.
;Authors: R&D Independent Team
;Source: www.pool-rnd.com

to elastic_base :xy
  pu setpos :xy pd

  make "mass 1
  make "speed 0
  to setspeed :s lock "speed ["speed := :s] end
  to ekin lock "speed [op 0.5 * :mass * :speed^2] end

  to cosv :v1 :v2
    output :v1,1 * :v2,1 + :v1,2 * :v2,2
  end
end

to elastic_m1 :xy
  use elastic_base :xy

  to onturtlecollision :turtle :vdir
    let "s1 :speed * cosv :vdir movedir
    let "s2 :speed @ :turtle * cosv :vdir movedir @ :turtle
    if :s1 > :s2 [output :s2 - :s1]
    output 0
  end

  let "n 0
  to onturtleinteraction :vdir :mag :drest
    if :mag = 0 [stop]
    lock "speed [
      let "s0 :speed
      let "sdir movedir
      let "sx :sdir,1 * :speed + :vdir,1 * :mag
      let "sy :sdir,2 * :speed + :vdir,2 * :mag
      "speed := sqrt :sx^2 + :sy^2
      "n += 1
      :sdir,1 := :sx / :speed
      :sdir,2 := :sy / :speed
      setmovedir :sdir alignh
      if (:drest > 0) && (:s0 > 0) [
        ifelse :n < 10
        [fd :drest * :speed / :s0]
        [(print :n "|collisions on| who)]
      ]
      "n -= 1
    ]
  end
end

to elastic :xy
  use elastic_base :xy

  to setmass :m lock "speed ["mass := :m] end

  to onturtlecollision :turtle :vdir
    let "s1 :speed * cosv :vdir movedir
    let "s2 :speed @ :turtle * cosv :vdir movedir @ :turtle
    if :s1 > :s2 [
      let "m2 :mass @ :turtle
      let "sx (:s1 * (:mass - :m2) + 2 * :m2 * :s2) / (:mass + :m2)
      output :mass * (:sx - :s1)
    ]
    output 0
  end

  let "n 0
  to onturtleinteraction :vdir :mag :drest
    if :mag = 0 [stop]
    lock "speed [
      let "s0 :speed
      let "sdir movedir
      let "sx (:sdir,1 * :mass * :speed + :vdir,1 * :mag) / :mass
      let "sy (:sdir,2 * :mass * :speed + :vdir,2 * :mag) / :mass
      "speed := sqrt :sx^2 + :sy^2
      "n += 1
      :sdir,1 := :sx / :speed
      :sdir,2 := :sy / :speed
      setheaddir :sdir
      if (:drest > 0) && (:s0 > 0) [
        ifelse :n < 10
        [fd :drest * :speed / :s0]
        [(print :n "|collisions on| who)]
      ]
      "n -= 1
    ]
  end
end

